'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Model = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _depd = require('depd');

var _depd2 = _interopRequireDefault(_depd);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _methods = require('./methods');

var _methods2 = _interopRequireDefault(_methods);

var _model = require('./model');

var _model2 = _interopRequireDefault(_model);

var _response = require('./response');

var _response2 = _interopRequireDefault(_response);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint no-use-before-define:0 */

var debug = (0, _debug2.default)('akita:client');
var deprecate = (0, _depd2.default)('akita:client');

var INSTANCES = {};

function resolve(key) {
  if (!INSTANCES[key]) {
    INSTANCES[key] = create();
  }
  return INSTANCES[key];
}

function create(options) {
  var client = function client(path) {
    var _class, _temp;

    return _temp = _class = function (_Model) {
      _inherits(AnonymousModel, _Model);

      function AnonymousModel() {
        _classCallCheck(this, AnonymousModel);

        return _possibleConstructorReturn(this, (AnonymousModel.__proto__ || Object.getPrototypeOf(AnonymousModel)).apply(this, arguments));
      }

      return AnonymousModel;
    }(_model2.default), _class.path = path, _class.client = client, _temp;
  };

  client._options = options || {};

  // 已经发送请求的数量
  client._count = 0;
  client.create = create;
  client.resolve = resolve;
  client.setOptions = function (opts) {
    client._options = Object.assign({}, client._options, opts);
  };

  function getFormDataClass() {
    var FormData = client._options.FormData;
    if (!FormData && (typeof window === 'undefined' ? 'undefined' : _typeof(window)) === 'object' && window.FormData) {
      FormData = window.FormData;
    } else if (!FormData && (typeof global === 'undefined' ? 'undefined' : _typeof(global)) === 'object' && global.FormData) {
      FormData = global.FormData;
    }
    return FormData;
  }

  client.request = function request(path, init, query, inspect) {
    init = Object.assign({}, init);
    if (init.params) {
      deprecate('akita.request(path, init): init.params deprecated, please use init.query instand.');
    }
    var queryParams = Object.assign({}, init.query || init.params);
    delete init.query;
    delete init.params;

    var queryString = _qs2.default.stringify(queryParams);
    if (queryString) {
      path += '?' + queryString;
    }

    if (init.body && _typeof(init.body) === 'object') {
      var body = init.body;
      var FormData = getFormDataClass();

      if (init.method === 'UPLOAD') {
        // 如果是UPLOAD
        init.method = 'POST';
        if (!FormData) {
          /* istanbul ignore next */
          throw new Error('Akita Error: Can not resolve FormData class when use upload method');
        }
        if (!(body instanceof FormData)) {
          // 自动构造 FormData
          var form = new FormData();
          Object.keys(body).forEach(function (name) {
            return form.append(name, body[name]);
          });
          init.body = form;
        }
      } else if (!FormData || !(body instanceof FormData)) {
        // 如果是普通POST请求，转换成JSON或urlencoded
        if (!init.headers) {
          init.headers = {};
        }
        if (!init.headers['Content-Type'] || init.headers['Content-Type'] === 'application/json') {
          init.headers['Content-Type'] = 'application/json';
          init.body = JSON.stringify(body);
        } else if (init.headers['Content-Type'] === 'application/x-www-form-urlencoded') {
          init.body = _qs2.default.stringify(body);
        } else {
          /* istanbul ignore next */
          throw new Error('Akita Error: Unsupported Content-Type ' + init.headers['Content-Type']);
        }
      }
    }

    if (client._options.init) {
      init = Object.assign({}, client._options.init, init);
      if (client._options.init.headers) {
        init.headers = Object.assign({}, client._options.init.headers, init.headers);
      }
    }

    var apiRoot = client._options.apiRoot;
    if (apiRoot) {
      if (apiRoot[apiRoot.length - 1] === '/' && path[0] === '/') {
        path = path.substring(1);
      } else if (apiRoot[apiRoot.length - 1] !== '/' && path[0] !== '/') {
        path = '/' + path;
      }
      path = apiRoot + path;
    }

    if (debug.enabled) {
      debug(init.method, path, JSON.stringify(init));
    }

    client.latest = Object.assign({}, init, {
      url: path
    });

    if (inspect) {
      return client.latest;
    }

    var fetch = client._options.fetch;

    if (!fetch) {
      if (typeof window !== 'undefined') {
        fetch = window.fetch;
      } else if (typeof global !== 'undefined') {
        fetch = global.fetch;
      }
    }

    client._count += 1;

    return new _response2.default(fetch, path, init, query);
  };

  _methods2.default.forEach(function (method) {
    client[method] = function (path, init, inspect) {
      init = init || {};
      init.method = method.toUpperCase();
      return client.request(path, init, null, inspect);
    };
  });

  return client;
}

exports.default = resolve('default');
exports.Model = _model2.default;