'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _methods = require('./methods');

var _methods2 = _interopRequireDefault(_methods);

var _query = require('./query');

var _query2 = _interopRequireDefault(_query);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug2.default)('akita:model');

var Model = function () {
  function Model(data, params) {
    _classCallCheck(this, Model);

    if (data) {
      Object.assign(this, data);
    }
    if (params) {
      Object.defineProperty(this, '__params', {
        value: Object.assign({}, params),
        writable: true,
        enumerable: false,
        configurable: true
      });
    }
  }

  /**
   * Create record
   * @param {Object} [data]
   * @returns {Query}
   */


  _createClass(Model, [{
    key: 'request',
    value: function request(path, init, inspect) {
      var _this = this;

      var M = this.constructor;
      var pk = M.pk || 'id';
      // $Flow indexer
      var id = this[pk];
      if (!id) {
        var method = init && init.method || 'GET';
        throw new Error('Can not get pk field (' + pk + ') for ' + method + ': \'' + path + '\'');
      }
      if (!path || path.startsWith('/')) {
        path = id + path;
      } else {
        path = id + '/' + path;
      }
      var fullPath = M.path + '/' + path;
      var matchs = fullPath.match(/:\w+/g);
      if (matchs) {
        init = Object.assign({}, init);
        var query = Object.assign({}, init.query);
        matchs.forEach(function (match) {
          var key = match.substr(1);
          if (!query.hasOwnProperty(key)) {
            if (_this.__params && _this.__params.hasOwnProperty(key)) {
              query[key] = _this.__params[key];
            } else if (_this.hasOwnProperty(key)) {
              // $Flow indexer
              query[key] = _this[key];
            } else if (_this.hasOwnProperty(key)) {
              // $Flow indexer
              query[key] = _this[key];
            }
          }
        });
        init.query = query;
      }
      return M.request(path, init, null, inspect);
    }
  }, {
    key: 'save',
    value: function save(init) {
      return this.request('', Object.assign({}, {
        method: 'PATCH',
        body: this.toJSON()
      }, init)).then(function () {
        return undefined;
      });
    }
  }, {
    key: 'remove',
    value: function remove(init) {
      return this.request('', Object.assign({}, {
        method: 'DELETE'
      }, init)).then(function () {
        return undefined;
      });
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      var json = {};
      for (var key in this) {
        if (this.hasOwnProperty(key)) {
          // $Flow indexer
          json[key] = this[key];
        }
      }
      return json;
    }
  }], [{
    key: 'create',
    value: function create(data) {
      var query = new _query2.default(this, 'create');
      query._data = data || {};
      return query;
    }

    /**
     * @example
     * // Update one record by id
     * await Blog.update(12, { title: 'hello' });
     * // Update multi records by filter
     * await Blog.update({ hot: true }).where('views').gt(100);
     * // Update one record by id and filter
     * await Blog.update(123,{ hot: true }).where('views').gt(100);
     * // Update multi records with limit and sort
     * await Blog.update({ hot: true }).sort('-createdAt').limit(100)
     * @param {Object|string|number} id
     * @param {Object} [data]
     * @returns {Query}
     */

  }, {
    key: 'update',
    value: function update(id, data) {
      var query = new _query2.default(this, 'update');
      if (data) {
        query._data = data;
      }
      /* istanbul ignore else */
      if (id && data) {
        query._id = id;
        query._data = data;
      } else if ((typeof id === 'undefined' ? 'undefined' : _typeof(id)) === 'object') {
        query._data = id;
      }
      return query;
    }

    /**
     * @example
     * // Remove one record by id
     * await Blog.remove(195);
     * // Remove multi records by filter
     * await Blog.remove().where('status', 800);
     * await Blog.remove({ status:800 });
     * // Remove one record by id and filter
     * await Blog.remove(195).where('status', 800);
     * // Remove multi records with limit and sort
     * await Blog.remove().where('status', 800).limit(100).sort('-createdAt');
     * @param {string|number|Object} [conditions]
     * @returns {Query}
     */

  }, {
    key: 'remove',
    value: function remove(conditions) {
      var query = new _query2.default(this, 'remove');
      if (conditions !== null && (typeof conditions === 'undefined' ? 'undefined' : _typeof(conditions)) === 'object') {
        query.where(conditions);
      } else {
        query._id = conditions;
      }
      return query;
    }

    /**
     * @example
     * await Blog.count();
     * await Blog.count().where('status', 800);
     * await Blog.count({ status:800 });
     * @param {Object} [conditions]
     * @returns {Query}
     */

  }, {
    key: 'count',
    value: function count(conditions) {
      var query = new _query2.default(this, 'count');
      if (conditions) {
        query.where(conditions);
      }
      return query;
    }

    /**
     * Find records without paging.
     * @example
     * await Blog.find().where('status', 800);
     * await Blog.find({ status:800 });
     * @param {Object} [conditions]
     * @returns {Query}
     */

  }, {
    key: 'find',
    value: function find(conditions) {
      var query = new _query2.default(this, 'find');
      if (conditions) {
        query.where(conditions);
      }
      return query;
    }

    /**
     * Find records with paging.
     * @example
     * await Blog.paginate().where('status', 800);
     * await Blog.paginate({ status:800 }).page(2);
     * @param {Object} [conditions]
     * @returns {Query}
     */

  }, {
    key: 'paginate',
    value: function paginate(conditions) {
      var query = new _query2.default(this, 'paginate');
      if (conditions) {
        query.where(conditions);
      }
      return query;
    }

    /**
     * @example
     * // Find one record by filter
     * var blog = await Blog.findOne().where('status', 800);
     * var blog = await Blog.findOne({status:800});
     * @param {Object} [conditions]
     * @returns {Query}
     */

  }, {
    key: 'findOne',
    value: function findOne(conditions) {
      var query = new _query2.default(this, 'findOne');
      query.limit(1);
      if (conditions) {
        /* istanbul ignore else */
        if ((typeof conditions === 'undefined' ? 'undefined' : _typeof(conditions)) === 'object') {
          query.where(conditions);
        } else {
          throw new Error('Akita Error: invalid params for Query#findOne()');
        }
      }
      return query;
    }

    /**
     * find record by id
     * @param {string|number} id
     * @returns {Query}
     */

  }, {
    key: 'findByPk',
    value: function findByPk(id) {
      var query = new _query2.default(this, 'findByPk');
      query._id = id;
      return query;
    }
  }, {
    key: 'request',
    value: function request(path, init, query, inspect) {
      var p = this.path || '';

      if (!p.endsWith('/') && path) {
        p += '/';
      }
      if (path.startsWith('/')) {
        path = path.substr(1);
      }
      path = p + path;

      if (path && init && (init.query || init.body)) {
        var queryParams = init.query && Object.assign({}, init.query);
        var bodyParams = init.body && _typeof(init.body) === 'object' && Object.assign({}, init.body);
        var params = {};
        var queryEmits = [];
        var bodyEmits = [];
        path = path.replace(/:\w+/g, function (match) {
          var key = match.substr(1); // trim :
          if (queryParams && queryParams.hasOwnProperty(key)) {
            queryEmits.push(key);
            params[key] = queryParams[key];
            return queryParams[key];
          } else if (bodyParams && bodyParams.hasOwnProperty(key)) {
            bodyEmits.push(key);
            params[key] = bodyParams[key];
            return bodyParams[key];
          }
          return match;
        });
        if (query && (queryEmits.length || bodyEmits.length)) {
          query._params = params;
        }
        if (queryParams && queryEmits.length) {
          queryEmits.forEach(function (key) {
            delete queryParams[key];
          });
          init = Object.assign({}, init, { query: queryParams });
        }
        if (bodyParams && bodyParams.length) {
          bodyEmits.forEach(function (key) {
            delete bodyParams[key];
          });
          init = Object.assign({}, init, { body: bodyParams });
        }
      }

      return this.client.request(path, init, query, inspect);
    }
  }]);

  return Model;
}();

exports.default = Model;


['upload'].concat(_methods2.default).forEach(function (method) {
  Model[method] = function (path, init, inspect) {
    debug(this.name + '.' + method, path, init || '');
    init = init || {};
    init.method = method.toUpperCase();
    return this.request(path, init, null, inspect);
  };

  // $Flow prototype indexer
  Model.prototype[method] = function (path, init, inspect) {
    var M = this.constructor;
    debug(M.name + '.prototype.' + method, path, init || '');
    init = init || {};
    init.method = method.toUpperCase();
    return this.request(path, init, inspect);
  };
});