'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _debug2 = require('debug');

var _debug3 = _interopRequireDefault(_debug2);

var _depd = require('depd');

var _depd2 = _interopRequireDefault(_depd);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug3.default)('akita:query');
var deprecate = (0, _depd2.default)('akita:query');

var Query = function () {

  /**
   * @param {Model} model
   * @param {string} op
   * @constructor
   */
  // 查询时，起到作用的路径参数列表
  function Query(model, op) {
    _classCallCheck(this, Query);

    this.model = model;
    this._filters = null;
    this._data = null;
    this._page = 0;
    this._limit = 0;
    this._sort = '';
    this._id = null;
    this._args = null;
    this._search = '';

    this._op = op;
    this._promise = null;
    this._lastField = '';
  }

  /**
   * Add custom http query args
   * @param {string|Object} args
   * @param {any} [value]
   * @returns {Query}
   */


  _createClass(Query, [{
    key: 'arg',
    value: function arg(args, value) {
      if (!this._args) {
        this._args = {};
      }
      if ((typeof args === 'undefined' ? 'undefined' : _typeof(args)) === 'object') {
        Object.assign(this._args, args);
      } else {
        this._args[args] = value;
      }
      return this;
    }

    /**
     * Add custom http query arg
     * @deprecated
     * @param {string|Object} args
     * @param {any} [value]
     * @returns {Query}
     */

  }, {
    key: 'param',
    value: function param(args, value) {
      deprecate('.param() deprecated, please use .arg() instand.');
      return this.arg(args, value);
    }

    /**
     * @example
     * var blogs = await akita('api/blog')
     *                   .find()
     *                   .where({ username: 'Liang' })
     *                   .where('status', 5)
     *                   .where('views')
     *                   .gt(100)
     *                   .sort('-createdAt')
     *                   .limit(10)
     *                   .page(1);
     * @param {string|Object} conditions
     * @param {any} [value]
     * @returns {Query}
     */

  }, {
    key: 'where',
    value: function where(conditions, value) {
      if (!this._filters) {
        this._filters = {};
      }
      if ((typeof conditions === 'undefined' ? 'undefined' : _typeof(conditions)) === 'object' && value === undefined) {
        this._filters = Object.assign(this._filters, conditions);
      } else if (typeof conditions === 'string') {
        if (value === undefined) {
          // where('foo')
          this._lastField = conditions;
        } else {
          this._filters[conditions] = value;
        }
      } else {
        /* istanbul ignore next */
        throw new Error('Akita Error: invalid params for Query#where()');
      }
      return this;
    }

    /**
     * Specifies search param
     * @param keyword
     * @returns {Query}
     */

  }, {
    key: 'search',
    value: function search(keyword) {
      this._search = keyword;
      return this;
    }
  }, {
    key: '__filter',
    value: function __filter(type, value) {
      if (!this._filters) {
        this._filters = {};
      }
      if (!this._lastField) {
        /* istanbul ignore next */
        throw new Error('Akita Error: you should invoke .where(string) before .' + type + '()');
      }

      if (type === 'eq') {
        this._filters[this._lastField] = value;
      } else {
        if (_typeof(this._filters[this._lastField]) !== 'object') {
          this._filters[this._lastField] = {};
        }
        this._filters[this._lastField]['$' + type] = value;
      }
      return this;
    }
  }, {
    key: 'eq',
    value: function eq(value) {
      return this.__filter('eq', value);
    }
  }, {
    key: 'lt',
    value: function lt(value) {
      return this.__filter('lt', value);
    }
  }, {
    key: 'lte',
    value: function lte(value) {
      return this.__filter('lte', value);
    }
  }, {
    key: 'gt',
    value: function gt(value) {
      return this.__filter('gt', value);
    }
  }, {
    key: 'gte',
    value: function gte(value) {
      return this.__filter('gte', value);
    }
  }, {
    key: 'limit',
    value: function limit(size) {
      this._limit = size;
      return this;
    }
  }, {
    key: 'page',
    value: function page(value) {
      this._page = value;
      return this;
    }
  }, {
    key: 'sort',
    value: function sort(value) {
      this._sort = value;
      return this;
    }

    /**
     * Execute the query.
     * @returns {Promise<*>}
     */

  }, {
    key: 'exec',
    value: function exec() {
      var _this = this;

      if (!this._promise) {
        this._debug();
        var init = this._createInit();
        var path = init.path;
        delete init.path;
        var p = void 0;
        // 处理请求
        switch (this._op) {
          case 'findOne':
            // findOne = find + limit 1
            p = this.model.request(path, init, this).then(function (results) {
              if (!results || !Array.isArray(results)) {
                throw new Error('Api error: GET ' + path + ' should return an object array.');
              }
              if (results.length) {
                return results[0];
              }
              return null;
            });
            break;
          case 'count':
            p = this.model.request(path, init, this).then(function (result) {
              if (!result || (typeof result === 'undefined' ? 'undefined' : _typeof(result)) !== 'object' || !result.hasOwnProperty('count')) {
                throw new Error('Api error: GET ' + path + ' should return an object with count property.');
              }
              return result.count;
            });
            break;
          default:
            p = this.model.request(path, init, this);
        }
        // 处理返回值
        var M = this.model;

        var createRecord = function createRecord(data) {
          return new M(data, _this._params);
        };

        switch (this._op) {
          case 'findByPk':
          case 'findOne':
          case 'create':
            p = p.then(function (object) {
              return object ? createRecord(object) : object;
            });
            break;
          case 'find':
            p = p.then(function (list) {
              if (!Array.isArray(list)) throw new Error('Api error: GET ' + path + ' should return an object array.');
              return list.map(createRecord);
            });
            break;
          case 'paginate':
            p = p.then(function (res) {
              if (res && res.results) {
                res.results = res.results.map(createRecord);
              }
              return res;
            });
            break;
          default:
            break;
        }
        this._promise = p;
      }
      return this._promise;
    }
  }, {
    key: 'inspect',
    value: function inspect() {
      this._debug();
      var init = this._createInit();
      var path = init.path;
      delete init.path;
      return this.model.request(path, init, this, true);
    }
  }, {
    key: '_debug',
    value: function _debug() {
      if (debug.enabled) {
        var str = '';
        var M = this.model;
        if (M.name === 'AnonymousModel') {
          str = 'Client("' + M.path + '").';
        } else {
          str = M.name + '.';
        }
        str += this._op;
        switch (this._op) {
          case 'findByPk':
            if (this._id === null) {
              throw new Error('id is not specified for findByPk');
            }
            str += '(' + JSON.stringify(this._id) + ')';
            break;
          case 'remove':
            if (this._id !== null) {
              str += '(' + JSON.stringify(this._id) + ')';
            } else {
              // remove multi
              str += '()';
            }
            break;
          case 'update':
            if (this._id) {
              str += '(' + JSON.stringify(this._id) + ', ' + JSON.stringify(this._data) + ')';
            } else {
              str += '(' + JSON.stringify(this._data) + ')';
            }
            break;
          case 'create':
            str += '(' + JSON.stringify(this._data) + ')';
            break;
          default:
            str += '()';
        }
        if (this._args) {
          var args = this._args;
          str += Object.keys(this._args).map(function (key) {
            return '.arg("' + key + '", ' + JSON.stringify(args[key]) + ')';
          }).join('');
        }
        if (this._filters) {
          str += '.where(' + JSON.stringify(this._filters) + ')';
        }
        if (this._search) {
          str += '.search("' + this._search + '")';
        }
        if (this._limit > 1) {
          str += '.limit(' + this._limit + ')';
        }
        if (this._page > 1) {
          str += '.page(' + this._page + ')';
        }
        if (this._sort) {
          str += '.sort(' + this._sort + ')';
        }
        debug(str);
      }
    }
  }, {
    key: '_createInit',
    value: function _createInit() {
      var init = {};
      init.method = 'GET';
      var path = '';

      var query = {};

      if (this._args) {
        var obj = this._args;
        Object.keys(obj).forEach(function (key) {
          query['_' + key] = obj[key];
        });
      }

      if (this._filters) {
        Object.assign(query, this._filters);
      }

      if (this._search) {
        query._search = this._search;
      }

      if (this._limit) {
        query._limit = this._limit;
      }

      if (this._page) {
        query._page = this._page;
      }

      if (this._sort) {
        query._sort = this._sort;
      }

      if (this._id && ['findByPk', 'remove', 'update'].indexOf(this._op) > -1) {
        path += '/' + encodeURIComponent(this._id);
      }

      if (this._data && ['create', 'update'].indexOf(this._op) > -1) {
        init.body = this._data;
      }

      switch (this._op) {
        case 'count':
          path += '/count';
          break;
        case 'paginate':
          path += '/paginate';
          break;
        case 'create':
          init.method = 'POST';
          break;
        case 'update':
          init.method = 'PATCH';
          break;
        case 'remove':
          init.method = 'DELETE';
          break;
        default:
        // find
        // findOne
        // findByPk
        // ...
      }
      init.path = path;
      init.query = query;
      return init;
    }
  }, {
    key: 'then',
    value: function then(onSuccess, onFail) {
      return this.exec().then(onSuccess, onFail);
    }
  }, {
    key: 'catch',
    value: function _catch(onFail) {
      return this.exec().catch(onFail);
    }
  }]);

  return Query;
}();

exports.default = Query;